'use strict';

const baiji = require('../');
const Controller = baiji.Controller;
const express = require('express');
const debug = require('debug')('baiji:examples:express');

const session = require('express-session');
const RedisStore = require('connect-redis')(session);

// Handle all uncaughtException avoid node instance crashing
process.on('uncaughtException', function(e) {
  debug('uncaughtException', e, e.stack);
});

class UsersCtrl extends Controller {
  constructor() {
    super();
    this.setName('users');

    // Add hooks
    this.beforeAction('loginRequired', { except: 'index' });
    this.beforeAction('checkAppExistance');
    this.beforeAction(function customBeforeAction(ctx, next) {
      debug('custom beforeAction called');
      next();
    }, { only: ['show'] });

    // this.afterErrorAction(function(ctx, next) {
    //   debug('afterError * executed.');
    //   debug('afterError =>', ctx.error, ctx.error.stack);
    //   ctx.done({ error: { name: ctx.error, stack: ctx.error.stack } });
    //   next();
    // });
  }

  initConfig() {
    return {
      index: { description: 'user list', route: { path: '/', verb: 'get' } },
      show: { description: 'user detail', route: { path: '/:id', verb: 'get' } }
    };
  }

  loginRequired(ctx, next) {
    debug('loginRequired executed');
    next();
  }

  checkAppExistance(ctx, next) {
    debug('checkAppExistance executed');
    next();
  }

  index(ctx, next) {
    debug('method executed', ctx.methodName);
    throw new Error('dddddd');
    ctx.done([
      { id: 1, username: 'felix' },
      { id: 2, username: 'jenny' }
    ]);
    next();
  }

  show(ctx, next) {
    debug('method executed', ctx.methodName);
    const session = ctx.req.session || {};
    session.abc = 1;
    this.handleApp();
    ctx.done({
      id: 1,
      username: 'felix'
    });
    next();
  }

  handleApp() {
    debug('UsersCtrl.prototype.handleApp called');
  }
}

let admin = baiji('admin');

// Main app
let app = baiji('myApp');

// Allow string to be splited as array by specific delimiters
app.set('adapterOptions.arrayItemDelimiters', ',');

// Enable X-Powered-By
app.enable('x-powered-by');

// Handle all undefined routes
app.define('404', {
  description: 'handle 404',
  route: { verb: 'all', path: '*' }
}, function(ctx, next) {
  debug('method executed', ctx.methodName);
  ctx.done({
    error: {
      name: '404',
      message: `no url available for ${ctx.method.toUpperCase()} ${ctx.path}`
    }
  });
  next();
});

app.before('*', function(ctx, next) {
  debug('before all executed.');
  next();
});

app.after('*', function(ctx, next) {
  debug('after all executed.');
  next();
});

app.afterError('*', function(ctx, next) {
  debug('afterError * executed.');
  debug('afterError =>', ctx.error, ctx.error.stack);
  ctx.done({ error: { name: ctx.error, stack: ctx.error.stack } });
  next();
});

admin.use(UsersCtrl, { mountPath: '/users' });

// 处理 Session
admin.use(session({
  name: `example`,
  store: new RedisStore({ port: '6379', host: '127.0.0.1' }),
  secret: 'fdsfdsfdfsadfdas',
  resave: false,
  saveUninitialized: false,
  cookie: { secure: false },
}), { name: 'session', desc: 'Session 处理中间件' });


// Mount User Controller
app.use(admin, { mountPath: '/admin' });


// Enable evaluator plugin
app.plugin('evaluator');

debug('app is listening on port 3005');
app.listen(3005);
